"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const solc = require("solc");
const fs = require("fs");
const fsx = require("fs-extra");
const pth = require("path");
const flattener_1 = require("@specron/flattener");
class Compiler {
    constructor(recipe) {
        this.output = {};
        this.input = {
            sources: Object.assign({}, (recipe ? recipe.sources : {})),
            language: recipe && recipe.language ? recipe.language : 'Solidity',
            settings: Object.assign({ evmVersion: 'istanbul', outputSelection: {
                    '*': {
                        '*': [
                            'ast', 'abi', 'metadata', 'evm.bytecode.object', 'evm.methodIdentifiers',
                        ],
                    },
                } }, (recipe ? recipe.settings : {})),
        };
        this.cwd = recipe && recipe.cwd ? recipe.cwd : process.cwd();
        this.reporter = recipe && recipe.reporter ? recipe.reporter : null;
    }
    source(...patterns) {
        return __awaiter(this, void 0, void 0, function* () {
            const flattener = new flattener_1.Flattener();
            flattener.source(...patterns);
            yield flattener.flatten();
            const keys = Object.keys(flattener.output.sources);
            keys.forEach(element => {
                this.input.sources[element] = {
                    content: flattener.output.sources[element]
                };
            });
            return keys.length;
        });
    }
    findImports(path) {
        path = path.indexOf('./') !== 0 ? `./node_modules/${path}` : path;
        return {
            contents: fs.readFileSync(path).toString(),
        };
    }
    compile() {
        if (this.reporter) {
            this.reporter.onCompileStart(this);
        }
        const input = JSON.stringify(this.input);
        this.output = JSON.parse(solc.compile(input, { import: this.findImports }));
        if (this.reporter) {
            this.reporter.onCompileEnd(this);
        }
        return !Array.isArray(this.output.errors);
    }
    clean() {
        try {
            Object.keys(this.output.contracts || {}).forEach((file) => {
                const sourcePath = this.normalizePath(file);
                const isModule = sourcePath.indexOf('./node_modules') === 0;
                if (isModule) {
                    return;
                }
                const matcher = new RegExp('(?<=^contract )(.*?)(?= |{|\n)', 'gm');
                const contracts = fs.readFileSync(sourcePath).toString().match(matcher);
                const json = this.output.contracts[file];
                Object.keys(json).forEach((contract) => {
                    if (contracts && contracts.indexOf(contract) === -1) {
                        delete this.output.contracts[file][contract];
                    }
                });
            });
        }
        catch (_a) { }
    }
    save(dist) {
        if (this.reporter) {
            this.reporter.onSaveStart(this);
        }
        const target = pth.resolve(this.cwd, dist);
        fsx.ensureDirSync(target);
        let count = 0;
        Object.keys(this.output.contracts || {}).forEach((file) => {
            const sourcePath = this.normalizePath(file);
            const isModule = sourcePath.indexOf('./node_modules') === 0;
            if (isModule) {
                return;
            }
            const fileName = pth.basename(sourcePath);
            const contractName = fileName.split('.').slice(0, -1).join('.');
            const destPath = pth.join(target, `${contractName}.json`);
            const json = this.output.contracts[file];
            Object.keys(json).forEach((contract) => {
                if (json[contract].metadata) {
                    json[contract].metadata = JSON.parse(json[contract].metadata);
                }
            });
            const data = JSON.stringify(json, null, 2);
            fs.writeFileSync(destPath, data);
            count++;
        });
        if (this.reporter) {
            this.reporter.onSaveEnd(this);
        }
        return count;
    }
    clear() {
        this.output = {};
        return this;
    }
    normalizePath(path) {
        return path.indexOf('./') !== 0 ? `./node_modules/${path}` : path;
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=compiler.js.map